
local detection = {}
 
local desc = 'CM-BOX端配置检测'
local cjson = require "cjson"
local GeneralFun = require ('data/rule/detection/GeneralFun')
local maskTable = {}

function detection.Detect()
	local webip = lua_getWebIP()
	local p = GeneralFun.getClassPointer()
	GeneralFun.writeFileUTF8("== "..desc.." ==")
	
	local seqs = {lua_getRequiredInfoSeqs(p)}
	GeneralFun.printSeqs(seqs)	
	if #seqs == 1 then	
		maskTable = {}
		local value = GeneralFun.getInfoBySeq(p,13)	if value == false then return end
		local boxSize = #value[webip]
		
		if boxSize == 0 then
			lua_addTip(p,"未配置云桌面，请检查云桌面配置情况")
			lua_addResult(p,2)
			lua_addScore(p,0)
			lua_afterDetect(p)
		else
			local bRelation = false
			local macs = ''			
			for i=1,boxSize do
				local tmIp = value[webip][i]["tmIp"]
				local tmMask = value[webip][i]["tmMask"]
				if string.len(tmIp) ~= 0 and string.len(tmMask) ~= 0 then
					maskTable[tmIp] = tmMask
				end
				if string.len(macs) > 0 then macs = macs .. ',' end
				macs = macs .. value[webip][i]['tmMac']
				if string.len(value[webip][i]['tmIp']) ~= 0 then bRelation = true end
			end			
			
			if not bRelation then 
				lua_addTip(p,"云课堂未关联任何云终端，请确认云课堂部署情况")
				lua_addResult(p,2)
				lua_addScore(p,0)
				lua_afterDetect(p)
			else
				local jsonRoot = {}
				jsonRoot[1]={}
				jsonRoot[1]['id']=17
				jsonRoot[1]['content']=''
				jsonRoot[2]={}
				jsonRoot[2]['id']=1001
				jsonRoot[2]['content']=macs
				jsonRoot[3]={}
				jsonRoot[3]['id']=5
				jsonRoot[3]['content']=''
				jsonRoot[4]={}
				jsonRoot[4]['id']=3007
				jsonRoot[4]['content']=''
				jsonRoot[5]={}
				jsonRoot[5]['id']=3
				jsonRoot[5]['content']=''
				jsonRoot[6]={}
				jsonRoot[6]['id']=4
				jsonRoot[6]['content']=''
				local jsonStr = cjson.encode(jsonRoot)
				lua_addRequiredInfoSeq(p,jsonStr)
			end			
		end		
	else	
		local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return end
		local wanIP = value[webip]["ip"]
		
		local value = GeneralFun.getInfoBySeq(p,5)	if value == false then return end
		local p1 = value[webip]["ip"]		
		local pos = string.len(p1)
		while(pos > 0)
		do
			local tmpIP = string.sub(p1,pos,pos)	
			if tmpIP == "." then
				break
			end
			pos = pos - 1
		end
		local p1_head = string.sub(p1,1,pos)
		GeneralFun.writeFileUTF8("p1_head:"..p1_head)
		
		local p2 = value[webip]["ip2"]
		local netmask = value[webip]["netmask"]
		
		local value = GeneralFun.getInfoBySeq(p,3007)	if value == false then return end
		local localips = ''
		local boxSize = #value["netinfo"]
		for i=1,boxSize do
			local localip = value["netinfo"][i]["ip"]
			localips = localips .. "，" .. localip
		end
		
		
		local value = GeneralFun.getInfoBySeq(p,3)	if value == false then return end
		local localMode = value[webip]["localMode"]
		local deployMode = string.lower( value[webip]["deployMode"])
		local value = GeneralFun.getInfoBySeq(p,17)	if value == false then return end
		local teacherIp = value[webip]["teacherIp"]
		local value = GeneralFun.getInfoBySeq(p,1001)	if value == false then return end
		local box_count = value["box_count"]		
		
		local bDetection1False = false
		if deployMode == "normal" then
			if localMode == "1" or localMode == "3" then
				if string.find(localips.."，",teacherIp.."，") == nil then
					bDetection1False = true
				end
			end
		end	
		if box_count == 0 then
			lua_addTip(p,"未扫描到CM_BOX端，请确认终端是否开机或终端系统版本正确")
			lua_addResult(p,3)
			lua_addScore(p,0)
			
			if bDetection1False then	
				lua_addTip(p,"该网络部署模式下需要将体检工具运行在教师机上检测")
				lua_addResult(p,3)
				lua_addScore(p,-10)
				lua_afterDetect(p)
				return
			end 
		else
			local bSEqual = true
			local bSEqual_ip = ""
			local bSEqual_warning = false
			local bSEqual_warning_ip = ""
			local bTEqual = true
			local boxSize = #value["box_set"]
			for i=1,boxSize do
				local box_serverIP = value["box_set"][i]['serverIP']
				local box_tearcherIP = value["box_set"][i]['teacherIP']
				local box_ip = value["box_set"][i]["StuIP"]
				local box_mask = maskTable[box_ip]
				if box_mask == nil then
					box_mask = ""
				end
				if box_serverIP ~= p1 then -- and box_serverIP ~= wanIP 
					if box_serverIP ~= wanIP then
						bSEqual = false
						if string.find(bSEqual_ip.."，",box_serverIP.."，") == nil then
							if string.len(bSEqual_ip)~=0 then
								bSEqual_ip = bSEqual_ip.."，"
							end
							bSEqual_ip = bSEqual_ip .. box_serverIP
						end
					else
						bSEqual_warning = true
						if string.find(bSEqual_warning_ip.."，",box_serverIP.."，") == nil then
							if string.len(bSEqual_warning_ip)~=0 then
								bSEqual_warning_ip = bSEqual_warning_ip.."，"
							end
							bSEqual_warning_ip = bSEqual_warning_ip .. box_serverIP						
						end
					end
				end
				
				GeneralFun.writeFileUTF8("box_tearcherIP:"..box_tearcherIP.." p1_head:"..p1_head.." localips:"..localips)
				if string.len(box_mask) > 0 then	
					if box_tearcherIP ~= "0.0.0.0" then
						if not lua_isSameSubnet(box_tearcherIP,p1,box_mask,netmask) then
							bTEqual = false
						end
						if localMode == "0" then
							if box_tearcherIP ~= teacherIp then
								bTEqual = false	
							end
						else					
							if string.find(localips.."，",box_tearcherIP.."，") == nil then
								bTEqual = false	
							end
						end
					end	
				end				
			end
			
			if not bSEqual then		
				local tmpTip = "CM学生端配置的云主机IP错误，请使用classmannager统一配置终端的主服务地址为RCD第一个网口IP"
				lua_addTip(p,tmpTip.."(详情见体检报告)",tmpTip..";设置有问题的server ip有:"..bSEqual_ip)
				lua_addResult(p,3)
				lua_addScore(p,-5)
			else
				lua_addResult(p,1)
			end
			
			if bSEqual_warning then
				local tmpTip = "RCD该网络部署模式下建议CM学生端云主机配置为RCD第一个网口IP"
				lua_addTip(p,tmpTip.."(详情见体检报告)",tmpTip..";设置有问题的server ip有:"..bSEqual_warning_ip)
				lua_addResult(p,2)
				lua_addScore(p,0)
			else
				lua_addResult(p,1)
			end
			
			if bDetection1False then	
				lua_addTip(p,"该网络部署模式下需要将体检工具运行在教师机上检测")
				lua_addResult(p,3)
				lua_addScore(p,-10)
				lua_afterDetect(p)
				return
			end
			
			if not bTEqual then
				local tip = 'CM学生端配置的教师机IP有误，教师机需设置为双IP，第二IP需配置与服务器第一网口IP同网段'
				if localMode == "0" then
					tip = 'CM学生端配置的教师机IP有误，需配置为' .. teacherIp
				else
					-- 填本机与服务器ip1同网段的本机ip
					-- p1 netmask
					
					local value = GeneralFun.getInfoBySeq(p,3007)	if value == false then return end					
					local boxSize = #value["netinfo"]
					for i=1,boxSize do
						local localip = value["netinfo"][i]["ip"]
						if lua_isSameSubnet(localip,p1,value["netinfo"][i]["mask"],netmask) then
							tip = 'CM学生端配置的教师机IP有误，需配置为' .. localip
							break
						end
					end
				end
			
				lua_addTip(p,tip)
				lua_addResult(p,3)
				lua_addScore(p,-5)
			else
				lua_addResult(p,1)
			end
		end
		
		lua_afterDetect(p)
		
	end
end
 
return detection