
local detection = {}
 
local desc = 'RCD堆叠部署场景检查'
local cjson = require "cjson"
local GeneralFun = require ('data/rule/detection/GeneralFun')
local _slaveHost = ''

-- 无连接或非千兆
function detection16_checkLink(p,slaveHost)	
	local webip = lua_getWebIP()
	
	local ips = GeneralFun.stringSplit(slaveHost,"\\")
	local value9 = GeneralFun.getInfoBySeq(p,9)	if value9 == false then return false end
	for i=1,#ips do
		local ip = ips[i]	
		
		--RCD服务器两个物理网口link为非千兆或网口未连接，分数 - 10分  提示5	
		local eth0 = string.lower(value9[ip]["eth0"])
		local eth1 = string.lower(value9[ip]["eth1"])
		-- state 0-ok 1-无连接 2-非千兆
		local state_eth0 = 0
		local state_eth1 = 0
		if eth0 == "na" then
			state_eth0 = 1
		elseif eth0 == "1000mb/s" then
			state_eth0 = 0
		else
			state_eth0 = 2
		end
		if eth1 == "na" then
			state_eth1 = 1
		elseif eth1 == "1000mb/s" then
			state_eth1 = 0
		else
			state_eth1 = 2
		end
		
		if state_eth0 == 0 and state_eth1 == 0 then
			lua_addResult(p,1)
		else
			-- 未连接
			local tip1 = ''
			if state_eth0 == 1 then
				tip1 = "物理网口1"
			end
			if state_eth1 == 1 then
				if string.len(tip1) ~= 0 then
					tip1 = tip1 .. "和"
				end
				tip1 = tip1 .. "物理网口2"
			end
			-- 云主机网口一未连接，请确认云主机网络连接状态
			if string.len(tip1) ~= 0 then
				lua_addResult(p,3)
				lua_addScore(p,-10)
				lua_addTip(p,"云主机"..ip..tip1.."未连接，请确认云主机网络连接状态")
			end		
			
			-- 非千兆
			local tip2 = ''
			if state_eth0 == 2 then
				tip2 = "物理网口1"			
			end
			if state_eth1 == 2 then
				if string.len(tip2) ~= 0 then
					tip2 = tip2 .. "和"
				end
				tip2 = tip2 .. "物理网口2"
			end
			-- 云主机网口二非千兆，请检查网线与交换机是否为千兆协商模式
			if string.len(tip2) ~= 0 then
				lua_addResult(p,3)
				lua_addScore(p,-10)
				lua_addTip(p,"云主机"..ip.."的"..tip2.."非千兆，请检查网线与交换机是否为千兆协商模式")
			end		
		end
		
	end
	
	return true
end

-- 9. 备机服务器ip冲突检测
function detection16_checkConflict(p,slaveHost)
	local webip = lua_getWebIP()
	
	local ips = GeneralFun.stringSplit(slaveHost,"\\")
	local value3 = GeneralFun.getInfoBySeq(p,3)	if value3 == false then return false end
	local value10 = GeneralFun.getInfoBySeq(p,10)	if value10 == false then return false end
	for i=1,#ips do
		local ip = ips[i]	
		local netcardMode = string.lower(value3[ip]["netcardMode"])
		local bBond = netcardMode == "bond"
		
		--检测到RCD服务器两个物理网口出现IP冲突情况 ，分数：-10分  提示6
		local ip1 = value10[ip]["ip"]
		local ip2 = value10[ip]["ip2"]
		local conflict_ip = ''
		if ip1 ~= "no" then
			conflict_ip = "物理网口1"
		end
		if not (bBond or ip2 == "no") then
			if string.len(conflict_ip) == 0 then
				conflict_ip = "物理网口2"
			else
				conflict_ip = "两个物理网口都"
			end
		end
		
		if string.len(conflict_ip) == 0 then
			lua_addResult(p,1)		
		else
			lua_addResult(p,3)
			lua_addScore(p,-10)
			lua_addTip(p,"云主机"..ip.."，"..conflict_ip.."地址冲突，请确认网络地址分配情况")
		end			
	end	
	return true
end

-- 8. 备机可用空间检测
function detection16_checkSpace(p,slaveHost)
	local webip = lua_getWebIP()
	
	local ips = GeneralFun.stringSplit(slaveHost,"\\")
	local value20 = GeneralFun.getInfoBySeq(p,20)	if value20 == false then return false end
	local value21 = GeneralFun.getInfoBySeq(p,21)	if value21 == false then return false end
	local value22 = GeneralFun.getInfoBySeq(p,22)	if value22 == false then return false end
	local value23 = GeneralFun.getInfoBySeq(p,23)	if value23 == false then return false end
	local error_20_ip = ''		-- 请检测是否镜像是否开启存储模式，并确定删除无用的差分文件 (5分)
	local error_21_ip = ''		-- 课程镜像空间接近上限，请检测是否需要备份并删除无用课程镜像 (3分)
	local warning_21_ip = ''	-- 课程镜像空间接近上限，请检测是否需要备份并删除无用课程镜像 (0分)
	local error_22_ip = ''		--云主机硬盘空间接近上限，请检测教师共享目录/学生共享目录，删除其中无用文件 (5分)
	local error_23_ip = ''		--共享目录大小接近上限，请删除共享目录中无用文件(5分)
	for i=1,#ips do
		local ip = ips[i]
		-- 请检测是否镜像是否开启存储模式，并确定删除无用的差分文件
		local percent = value20[ip]["percent"]
		local per = tonumber(string.sub(percent,0,string.len(percent)-1))
		if per > 95 then
			if string.len(error_20_ip) ~= 0 then
				error_20_ip = error_20_ip .. "，"
			end
			error_20_ip = error_20_ip .. ip			
		end
		-- 课程镜像空间接近上限，请检测是否需要备份并删除无用课程镜像 
		percent = value21[ip]["percent"]
		per = tonumber(string.sub(percent,0,string.len(percent)-1))
		if per > 98 then
			if string.len(error_21_ip) ~= 0 then
				error_21_ip = error_21_ip .. "，"
			end
			error_21_ip = error_21_ip .. ip
		elseif per > 90 then
			if string.len(warning_21_ip) ~= 0 then
				warning_21_ip = warning_21_ip .. "，"
			end
			warning_21_ip = warning_21_ip .. ip
		end
		--云主机硬盘空间接近上限，请检测教师共享目录/学生共享目录，删除其中无用文件
		percent = value22[ip]["percent"]
		per = tonumber(string.sub(percent,0,string.len(percent)-1))
		if per > 90 then
			if string.len(error_22_ip) ~= 0 then
				error_22_ip = error_22_ip .. "，"
			end
			error_22_ip = error_22_ip .. ip
		end
		--共享目录大小接近上限，请删除共享目录中无用文件
		percent = value23[ip]["percent"]
		per = tonumber(string.sub(percent,0,string.len(percent)-1))
		if per > 83 then
			if string.len(error_23_ip) ~= 0 then
				error_23_ip = error_23_ip .. "，"
			end
			error_23_ip = error_23_ip .. ip
		end
	end	
	
	if string.len(error_20_ip) == 0 then
		lua_addResult(p,1)
	else
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"请检测是否镜像是否开启存储模式，并确定删除无用的差分文件。有此问题的备机IP："..error_20_ip)
	end
	
	if string.len(error_21_ip) == 0 then
		lua_addResult(p,1)
	else
		lua_addResult(p,3)
		lua_addScore(p,-3)
		lua_addTip(p,"课程镜像空间接近上限，请检测是否需要备份并删除无用课程镜像。有此问题的备机IP："..error_21_ip)
	end
	
	if string.len(warning_21_ip) == 0 then
		lua_addResult(p,1)
	else
		lua_addResult(p,2)
		lua_addScore(p,0)
		lua_addTip(p,"课程镜像空间接近上限，请检测是否需要备份并删除无用课程镜像。有此问题的备机IP："..warning_21_ip)
	end
		
	if string.len(error_22_ip) == 0 then
		lua_addResult(p,1)
	else
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"云主机硬盘空间接近上限，请检测教师共享目录/学生共享目录，删除其中无用文件。有此问题的备机IP："..error_22_ip)
	end
		
	if string.len(error_23_ip) == 0 then
		lua_addResult(p,1)
	else
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"共享目录大小接近上限，请删除共享目录中无用文件。有此问题的备机IP："..error_23_ip)
	end	
	
	return true
end

function detection.Detect()
	local webip = lua_getWebIP()
	local p = GeneralFun.getClassPointer()
	GeneralFun.writeFileUTF8("== "..desc.." ==")

	local seqs = {lua_getRequiredInfoSeqs(p)}
	GeneralFun.printSeqs(seqs)	
	if #seqs == 1 then	
		local value = GeneralFun.getInfoBySeq(p,24)		if value == false then return end
		local slaveHost = value[webip]["slaveHost"]
		local needIP = ''
		for i=1,#slaveHost do
			local tmpIP = slaveHost[i]		
			local bConnect = lua_getConnectState(tmpIP)
			if not bConnect then
				lua_addTip(p,"主控云主机无法与从机通讯，请确认从机端口与IP设置是否正常")
				lua_addResult(p,3)
				lua_addScore(p,-10)
				lua_afterDetect(p)
				return
			end
			if string.len(needIP) ~= 0 then needIP = needIP .. "\\" end
			needIP = needIP .. tmpIP
		end	
		_slaveHost = needIP
		GeneralFun.writeFileUTF8("slaveHost:<".._slaveHost..">")
		
		if string.len(_slaveHost) == 0 then -- 无从机，移除该项检测
			lua_removeDetection(p)
			lua_afterDetect(p)
			return
		else	--	有从机，增加需求信息
			local requestSeq = 1
			local jsonRoot = {}
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=9
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=2
			jsonRoot[requestSeq]['content']=_slaveHost..'\\'..webip
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=28
			jsonRoot[requestSeq]['content']=_slaveHost..'\\'..webip
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=27
			jsonRoot[requestSeq]['content']=webip
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=1
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=15
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=16
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=11
			jsonRoot[requestSeq]['content']=_slaveHost..'\\'..webip
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=20
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=21
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=22
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=23
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=3
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=10
			jsonRoot[requestSeq]['content']=_slaveHost
			requestSeq = requestSeq + 1
			
			jsonRoot[requestSeq]={}
			jsonRoot[requestSeq]['id']=33
			jsonRoot[requestSeq]['content']=_slaveHost..'\\'..webip
			requestSeq = requestSeq + 1
		
			local jsonStr = cjson.encode(jsonRoot)
			lua_addRequiredInfoSeq(p,jsonStr)
			return
		end
		
	end
	
	-- 获取足够信息，开始验证	
	
	--1、主控无法PING通从机（可以是多台从机）两个物理网口中任何一个口，分数： - 10分 提示：主控云主机无法与从机通讯，请确认从机端口与IP设置是否正常
	--主控PING通从机是否通	// 修改为取两个网卡是否有na    9.	RCD服务器两个物理网卡是否link为千兆	
	local bRunOK = detection16_checkLink(p,_slaveHost)
	if not bRunOK then
		return
	end
	-- 检测1和3合并
	--[[
	local value = GeneralFun.getInfoBySeq(p,9)	if value == false then return end
	local ips = GeneralFun.stringSplit(_slaveHost,"\\")
	local bOK = true
	for i=1,#ips do
		local tmpIP = ips[i]
		local tmpObj = value[tmpIP]
		local tip = ''
		local eth0 = string.lower( tmpObj["eth0"])
		local eth1 = string.lower( tmpObj["eth1"])
		if eth0 == "na" then
			tip = "物理网口1" 
		end
		if eth1 == "na" then
			if string.len(tip) == 0 then
				tip = "物理网口2"
			else
				tip = "两个物理网口都"
			end
		end
		
		-- 1 提示：云主机xxx.xxx.xxx物理网口1/2无法正常连接		
		if string.len(tip) ~= 0 then
			lua_addTip(p,"云主机"..tmpIP..tip.."无法正常连接")
			lua_addResult(p,3)
			lua_addScore(p,-10)
		else
			lua_addResult(p,1)		
		end
	end
	]]
	
	-- 3、确认堆叠环境中所有云主机网口工作在千兆转发模式下，若出现某台云主机端口协商非千兆模式，分数 - 10分，并给出具体哪台云主机物理接口协商模式有误
	--[[
	local value = GeneralFun.getInfoBySeq(p,9)	if value == false then return end
	local ips = GeneralFun.stringSplit(_slaveHost,"\\")
	local errorIP = ''
	local bOK = true
	for i=1,#ips do
		local tmpIP = ips[i]
		local tmpObj = value[tmpIP]
		
		local eth0 = string.lower( tmpObj["eth0"])
		local eth1 = string.lower( tmpObj["eth1"])
		if eth0 ~= "1000mb/s" or eth1 ~= "1000mb/s" then
			bOK = false
			if string.len(errorIP) ~=0 then errorIP = errorIP.."\\" end
			errorIP = errorIP .. tmpIP
			break
		end
	end
	
	if not bOK then
		lua_addTip(p,"云主机"..errorIP.."，物理网口协商为非千兆，请检查该云主机网线与交换机协商状态")
		lua_addResult(p,3)
		lua_addScore(p,-10)
	else
		lua_addResult(p,1)		
	end
	]]
	
	
	--2、连接所有从机，获取从机版本信息与主控版本信息，若不一致，分数：-20分 提示：堆叠环境中云主机版本不一致，请确认所有云主机版本
	local value = GeneralFun.getInfoBySeq(p,2)	if value == false then return end
	local ips = GeneralFun.stringSplit(_slaveHost.."\\"..webip,"\\")
	local lastVer = ''
	local bOK = true
	for i=1,#ips do
		local tmpIP = ips[i]
		local tmpObj = value[tmpIP]
		local tmpVer = tmpObj["version"]
		if string.len(lastVer) ~= 0 and lastVer ~= tmpVer then
			bOK = false
			break
		end
		lastVer = tmpVer
	end
	
	if not bOK then
		lua_addTip(p,"堆叠环境中云主机版本不一致，请确认所有云主机版本")
		lua_addResult(p,3)
		lua_addScore(p,-20)
	else
		lua_addResult(p,1)		
	end
	
	-- new 校验主控被控机角色配置是否正确，当前服务器角色设置为主控才进行镜像同步检测
	local value33 = GeneralFun.getInfoBySeq(p,33)	if value33 == false then return end
	local ips = GeneralFun.stringSplit(_slaveHost.."\\"..webip,"\\")
	local error_role_ip1 = ''	--	应该设置为主控的，设置错误的ip
	local error_role_ip2 = ''	--	应该设置为备控的，设置错误的ip
	for i=1,#ips do
		local tmpIP = ips[i]
		local tmpObj = value33[tmpIP]
		local tmpRole = tmpObj["role"]
		if tmpIP == webip then
			if tmpRole ~= "0" then
				-- 当前角色未设置为主控，请确认是否配置错误
				error_role_ip1 = tmpIP
			end
		else
			if tmpRole == "0" then
				-- 当前角色未设置为备控，请确认是否配置错误
				if string.len(error_role_ip2) > 0 then
					error_role_ip2 = error_role_ip2 .. "\\"
				end
				error_role_ip2 = error_role_ip2 .. tmpIP
			end
		end
	end
	local errorTip = ''
	if string.len(error_role_ip1) > 0 then
		errorTip = '当前服务器角色未设置为主控'
	end
	if string.len(error_role_ip2) > 0 then
		if string.len(errorTip) > 0 then
			errorTip = errorTip .. "，"
		end
		errorTip = errorTip .. "从机IP：" .. error_role_ip2 .. '工作角色未配置为"被控云主机"'
	end
	if string.len(errorTip) > 0 then
		errorTip = errorTip .. '，可在"系统"->"网络配置"中修改该配置'
		lua_addTip(p,errorTip)
		lua_addResult(p,3)
		lua_addScore(p,-10)
	else
		lua_addResult(p,1)	
	end
	-- 当前服务器角色未设置为主控，不进行镜像同步检测
	if string.len(error_role_ip1) == 0 then		
		-- 4、校验主控与从机课程镜像目录中base文件大小是否一致，若不一致，分数：-10分，并给出提示未同步该课程镜像
		local value = GeneralFun.getInfoBySeq(p,28)	if value == false then return end
		local ips = GeneralFun.stringSplit(_slaveHost.."\\"..webip,"\\")
		local baseObj = value[webip]
		local error_info = {}		
		
		for k,v in pairs(baseObj) do			
			for i=1,#ips do
				local tmpIP = ips[i]
				local tmpObj = value[tmpIP]
				local tmpSize = tmpObj[k]
				
				if v ~= tmpSize then
					if error_info[tmpIP] == nil then
						error_info[tmpIP] = '"'..k
					else
						error_info[tmpIP] = error_info[tmpIP] .. '","' .. k
					end
				end
			end
		end		
		
		local tip1 = ''
		local tip2 = ''
		for error_ip,error_name in pairs(error_info) do
			if string.len(tip1) == 0 then
				tip1 = '从机IP：'
			else
				tip1 = tip1 .. "\\"
			end
			tip1 = tip1 .. error_ip
			
			if string.len(tip2) ~= 0 then
				tip2 = tip2 .. ";"
			end
			tip2 = tip2 .. "从机IP：" .. error_ip .. "的" .. lua_charToUTF8(error_name) .. '"'
		end
		
		if string.len(tip1) ~= 0 then
			tip1 = tip1 .. "，课程镜像未同步，请立刻同步(详情见体检报告)"
			tip2 = tip2 .. "课程镜像未同步，请立刻同步"
			lua_addTip(p,tip1,tip2)
			lua_addResult(p,3)
			lua_addScore(p,-10)
		else
			lua_addResult(p,1)	
		end
	end	
	
	--	5、校验主控与从机共享目录中.macinfo.ini文件大小是否一致，若不一致，分数 - 10分，并给出提示同步共享目录
	local value = GeneralFun.getInfoBySeq(p,27)	if value == false then return end
	local szShare = string.lower(cjson.encode(value[webip]))
	if string.find(szShare,"yes") ~= nil then
		lua_addResult(p,1)	
	elseif string.find(szShare,"no") ~= nil then
		lua_addTip(p,"共享目录未同步，请点击同步共享目录")
		lua_addResult(p,3)
		lua_addScore(p,-10)
	else
		GeneralFun.failedDection()
		return
	end
	
	-- 6、校验从机是否配置云桌面、DHCP，若配置，分数： - 10分；
	local value1 = GeneralFun.getInfoBySeq(p,1)	if value1 == false then return end
	local value15 = GeneralFun.getInfoBySeq(p,15)	if value15 == false then return end
	local value16 = GeneralFun.getInfoBySeq(p,16)	if value16 == false then return end
	local errorContent = ''
	local errorIP = ''
	local ips = GeneralFun.stringSplit(_slaveHost,"\\")
	local bOK = true
	for i=1,#ips do
		local tmpIP = ips[i]
		-- 云桌面配置信息
		if value1[tmpIP]["vmAmount"] ~= "0" then
			bOK = false
			if string.find(errorIP.."，",tmpIP.."，") == nil then
				if string.len(errorIP) ~= 0 then
					errorIP = errorIP .. "，"
				end
				errorIP = errorIP .. tmpIP
			end
			if string.find(errorContent,"云桌面配置信息") == nil then
				if string.len(errorContent) ~= 0 then
					errorContent = errorContent.."、"
				end
				errorContent = errorContent .. "云桌面配置信息"
			end
		end
		-- DHCP服务
		if string.lower(value15[tmpIP]["dhcpStatus"]) == "on" then
			bOK = false
			if string.find(errorIP.."，",tmpIP.."，") == nil then
				if string.len(errorIP) ~= 0 then
					errorIP = errorIP .. "，"
				end
				errorIP = errorIP .. tmpIP
			end
			if string.find(errorContent,"DHCP服务") == nil then
				if string.len(errorContent) ~= 0 then
					errorContent = errorContent.."、"
				end
				errorContent = errorContent .. "DHCP服务"				
			end
		end
		-- 镜像分发
		if string.lower(value16[tmpIP]["pxeStatus"]) == "on" then
			bOK = false
			if string.find(errorIP.."，",tmpIP.."，") == nil then
				if string.len(errorIP) ~= 0 then
					errorIP = errorIP .. "，"
				end
				errorIP = errorIP .. tmpIP
			end
			if string.find(errorContent,"镜像分发") == nil then
				if string.len(errorContent) ~= 0 then
					errorContent = errorContent.."、"
				end
				errorContent = errorContent .. "镜像分发"				
			end
		end
	end
	
	if not bOK then
		lua_addTip(p,"云主机"..errorIP.."有多余配置，请关闭或删除从机的"..errorContent)
		lua_addResult(p,3)
		lua_addScore(p,-10)
	else
		lua_addResult(p,1)	
	end
	
	-- 7、主从机客户信息一致性校验（客户名、客户类别、教室），必填项必须一致，若配置不一致，分数： - 3分  提示：请确认客户信息一致性
	local value = GeneralFun.getInfoBySeq(p,11)	if value == false then return end
	local ips = GeneralFun.stringSplit(_slaveHost.."\\"..webip,"\\")
	local lastInfo = ''
	local bOK = true
	for i=1,#ips do
		local tmpIP = ips[i]
		local tmpObj = value[tmpIP]
		local info = tmpObj["name"] .. tmpObj["customer_type"] .. tmpObj["classroom"]
		if string.len(lastInfo)~=0 and lastInfo~=info then
			bOK = false
			break
		end
		lastInfo = info
	end
	
	if not bOK then
		lua_addTip(p,"请确认客户信息一致性")
		lua_addResult(p,3)
		lua_addScore(p,-10)
	else
		lua_addResult(p,1)	
	end
	
	GeneralFun.writeFileUTF8("_slaveHost:".._slaveHost)
	-- 8 备机可用空间检测
	bRunOK = detection16_checkSpace(p,_slaveHost)
	if not bRunOK then
		return
	end
	
	-- 9. 备机服务器ip冲突检测
	bRunOK = detection16_checkConflict(p,_slaveHost)
	if not bRunOK then
		return
	end
	
	lua_afterDetect(p)	
end
 
return detection