local detection = {}

local desc = '教师机健康检查'
local cjson = require "cjson"
local GeneralFun = require ('data/rule/detection/GeneralFun')

--1、检测工具是否运行在教师机上
function detection21_isTeacherTM(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,17)	if value == false then return false end
	local teacherIp = value[webip]["teacherIp"]
	
	local value = GeneralFun.getInfoBySeq(p,3007)	if value == false then return false end
	local bEqual = false
	local boxSize = #value["netinfo"]
	for i=1,boxSize do
		local localip = value["netinfo"][i]["ip"]
		if teacherIp == localip then
			bEqual = true
			break
		end
	end
	if not bEqual then
		lua_addResult(p,2)
		lua_addScore(p,0)
		lua_addTip(p,"当前检测设备非教师机，无法进行相关检测")
		lua_afterDetect(p)
		return false
	else
		lua_addResult(p,1)		
		return true
	end
end

-- 检测网页配置的教师机IP是否正确 5分
-- 网络模式0下教师机IP和物理网口1同网段 ；网络模式1下教师机IP和上网地址同网段；网络模式3下教师机IP和192.168.123.0\24同网段
function detection21_isTeacherIPError(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,17)	if value == false then return false end
	local teacherIp = value[webip]["teacherIp"]	
	
	local value = GeneralFun.getInfoBySeq(p,3)	if value == false then return end
	local deployMode = string.lower( value[webip]["deployMode"])
	
	if deployMode == "normal" then
		local localMode = value[webip]["localMode"]
		if localMode == "1" then	-- 网络模式1下教师机IP和上网地址同网段；			
			local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return false end
			local wanIP = value[webip]["ip"]
			local wanMask = value[webip]["netmask"]		
			
			if lua_isSameSubnet(teacherIp,wanIP,wanMask,wanMask) then		
				lua_addResult(p,1)		
				return false
			else
				lua_addResult(p,3)
				lua_addScore(p,-5)
				lua_addTip(p,"该网络模式下云主机管理页面的教师机IP需配置与上网地址同网段")
				lua_afterDetect(p)
				return true
			end
		elseif localMode == "3" then	-- 网络模式3下教师机IP和192.168.123.0\24同网段
			if lua_isSameSubnet(teacherIp,"192.168.123.0","255.255.255.0","255.255.255.0") then			
				lua_addResult(p,1)	
				return false
			else
				lua_addResult(p,3)
				lua_addScore(p,-5)
				lua_addTip(p,"该网络模式下云主机管理页面的教师机IP需配置192.168.123.0/24网段IP")
				lua_afterDetect(p)
				return true
			end
		elseif localMode == "0" then	-- 网络模式0下教师机IP和物理网口1同网段 		
			local value = GeneralFun.getInfoBySeq(p,5)		if value == false then return false end
			local ip1 = value[webip]["ip"]
			local netmask = value[webip]["netmask"]		
			
			if lua_isSameSubnet(teacherIp,ip1,netmask,netmask) then
				lua_addResult(p,1)
				return false
			else
				lua_addResult(p,3)
				lua_addScore(p,-5)
				lua_addTip(p,"该网络模式下云主机管理页面的教师机IP需配置与物理网口同网段")
				lua_afterDetect(p)
				return true
			end
		end
	end	
	
	return false
end

--2、RCD终端私网、虚机校园网模式：教师机须配置双IP，主IP与上网地址同网段，第二IP为终端私网同网段IP，教师机主IP与RCD配置的教师机IP一致
--2、提示：教师机须设置为双IP、单网关，网关设置为校园网网关，主IP与上网地址同网段，第二IP设置为与云主机物理网口同网段IP
function detection21_mode1(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,17)	if value == false then return false end
	local teacherIp = value[webip]["teacherIp"]
	
	local value = GeneralFun.getInfoBySeq(p,5)		if value == false then return false end
	local ip1 = value[webip]["ip"]
	local ip2 = value[webip]["ip2"]
	local netmask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return false end
	local wanIP = value[webip]["ip"]
	local wanGateway = value[webip]["gateway"]
	local wanMask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,3007)		if value == false then return false end
	local boxSize = #value["netinfo"]
	local b1 = string.find(value["netinfo"][1]["gateway"],",") == nil -- 单网关
	if boxSize < 2 then
		local tip = "教师机需设置为双IP"
		if not b1 then 
			tip = tip .. ",单网关，网关设置为校园网网关" 
		end
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,tip)
		return true
	end
	local b2 = false -- 主ip配置
	local b3 = false -- 从ip配置
	for i=1,boxSize do		
		if value["netinfo"][i]["ip"] == teacherIp then -- 主ip
			if lua_isSameSubnet(value["netinfo"][i]["ip"],wanIP,value["netinfo"][i]["mask"],wanMask) then
				b2 = true
			end
		else	-- 从ip
			if lua_isSameSubnet(value["netinfo"][i]["ip"],ip1,value["netinfo"][i]["mask"],netmask)
				or lua_isSameSubnet(value["netinfo"][i]["ip"],ip2,value["netinfo"][i]["mask"],netmask) then
				b3 = true
			end
			
		end
	end
	local tip = "" 
	if not b1 then
		tip = tip .. "教师机需设置为单网关，网关设置为校园网网关" 
	end
	if not b2 then		
		if string.len(tip) == 0 then  
			tip = "教师机"
		else
			tip = tip .. "，"
		end
		tip = tip .. "主ip需设置为上网地址同网段"
	end
	if not b3 then		
		if string.len(tip) == 0 then  
			tip = "教师机"
		else
			tip = tip .. "，"
		end
		tip = tip .. "第二IP需设置为与云主机物理网口同网段IP"
	end
	
	if tip ~= "" then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,tip)
	else
		lua_addResult(p,1)
	end
	
	return true
end

-- 3、RCD终端、虚机均私网模式：检测教师机主IP为192.168.123.0/24网段，网关为192.168.123.1，第二IP与云主机物理网口同网段IP，若不一致，分数：-5分。
-- 3、提示：教师机须设置为双IP、单网关，主IP为192.168.123.0/24网段，网关为192.168.123.1，第二IP与云主机物理网口同网段IP
function detection21_mode3(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,17)	if value == false then return false end
	local teacherIp = value[webip]["teacherIp"]
	if string.find(teacherIp,"192.168.123.") == 1 then --and string.find(teacherMask,"255.255.255.0") == 1 then	
		lua_addResult(p,1)
	else
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"RCD服务器的教师端IP需设置为192.168.123.0/24网段IP")
		return true
	end 
	
	local value = GeneralFun.getInfoBySeq(p,5)		if value == false then return false end
	local ip1 = value[webip]["ip"]
	local ip2 = value[webip]["ip2"]
	local netmask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return false end
	local wanIP = value[webip]["ip"]
	local wanGateway = value[webip]["gateway"]
	local wanMask = value[webip]["netmask"]
	local value = GeneralFun.getInfoBySeq(p,3007)		if value == false then return false end
	local boxSize = #value["netinfo"]
	local b1 = string.find(value["netinfo"][1]["gateway"],",") == nil -- 单网关
	local b2 = true--string.find(value["netinfo"][1]["gateway"],"192.168.123.1") ~= nil -- 网关为192.168.123.1
	if boxSize < 2 then
		local tip = "教师机需设置为双IP"
		if not b1 then
			tip = tip .. "，单网关"
		end
		if not b2 then
			tip = tip .. "，网关为192.168.123.1"
		end
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,tip)
		return true
	end
	local b3 = false -- 主ip配置
	local b4 = false -- 从ip配置
	for i=1,boxSize do		
		if value["netinfo"][i]["ip"] == teacherIp then -- 主ip
			if string.find(value["netinfo"][i]["ip"],"192.168.123.") == 1
				and  string.find(value["netinfo"][i]["mask"],"255.255.255.0") == 1 then
				b3 = true
			end
		else	-- 从ip
			if lua_isSameSubnet(value["netinfo"][i]["ip"],ip1,value["netinfo"][i]["mask"],netmask)
				or lua_isSameSubnet(value["netinfo"][i]["ip"],ip2,value["netinfo"][i]["mask"],netmask) then
				b4 = true
			end			
		end
	end
	local tip = "" 
	if not b1 then
		tip = tip .. "教师机需设置为单网关" 
	end
	if not b2 then
		if string.len(tip) == 0 then  
			tip = "教师机需设置"
		else
			tip = tip .. "，"
		end
		tip = tip .. "网关为192.168.123.1" 
	end
	
	if not b3 then
		if string.len(tip) ~= 0 then
			tip = tip .. "，" 
		else
			tip = "教师机"
		end
		tip = tip .. "主ip需设置为192.168.123.0/24网段IP"
	end
	if not b4 then
		if string.len(tip) ~= 0 then
			tip = tip .. "，" 
		else
			tip = "教师机"
		end
		tip = tip .. "第二IP需设置为与云主机物理网口同网段IP"
	end
	
	if tip ~= "" then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,tip)
	else
		lua_addResult(p,1)
	end	
	return true
end

function detection.Detect()
	local webip = lua_getWebIP()
	local p = GeneralFun.getClassPointer()
	GeneralFun.writeFileUTF8("== "..desc.." ==")
	
	--1、检测工具是否运行在教师机上
	if not detection21_isTeacherTM(p) then
		return
	end
	
	-- 网页配置的教师机ip是否有错误
	if detection21_isTeacherIPError(p) then
		return
	end
	
	--2\3
	local value = GeneralFun.getInfoBySeq(p,3)	if value == false then return end
	local deployMode = string.lower( value[webip]["deployMode"])
	
	if deployMode == "normal" then
		local localMode = value[webip]["localMode"]
		if localMode == "1" then
			local bRunOK = detection21_mode1(p)
			if not bRunOK then
				return
			end
		elseif localMode == "3" then
			local bRunOK = detection21_mode3(p)
			if not bRunOK then
				return
			end
		end
	end	
	
	--4.健康检查工具运行在教师机上，检测管理员工具admin-tool是否与当前RCD版本一致，若不一致，分数：-5分。
	local value = GeneralFun.getInfoBySeq(p,30)	if value == false then return end
	local ser_at_ver = value[webip]["version"]
	local value = GeneralFun.getInfoBySeq(p,3008)	if value == false then return end
	local at_ver = value["version"]
	if lua_compareVer(ser_at_ver,at_ver) ~= 1 then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"请升级管理员工具")
	else
		lua_addResult(p,1)
	end
		
	lua_afterDetect(p)
end
 
return detection