
local detection = {}
 
local desc = '生成报告'
local cjson = require "cjson"
local GeneralFun = require ('data/rule/detection/GeneralFun')

function detection.Detect()
	local webip = lua_getWebIP()
	local p = GeneralFun.getClassPointer()
	GeneralFun.writeFileUTF8("== "..desc.." ==")
	
	local jsonRoot = {}
	
	local value = GeneralFun.getInfoBySeq(p,11)		if value == false then return end
	local name = value[webip]["name"]
	jsonRoot["name"] = string.len(name)~=0 and name or "NA"
	local classroom = value[webip]["classroom"]
	jsonRoot["cu"] = string.len(classroom)~=0 and classroom or "NA"
	
	local value = GeneralFun.getInfoBySeq(p,7)		if value == false then return end
	local ip = value[webip]["ip"]
	local ip2 = value[webip]["ip2"]
	
	local value = GeneralFun.getInfoBySeq(p,25)		if value == false then return end
	local rcdsn = ""
	local slavercdsn = ""
	local hostSn = value[webip]["hostSn"]	
	for k,v in pairs(hostSn) do
		for tmp_ip,tmp_rcdsn in pairs(v) do
			if tmp_ip == webip or tmp_ip == ip or tmp_ip == ip2 then
				rcdsn = tmp_rcdsn
			else
				if string.len(slavercdsn) ~= 0 then
					slavercdsn = slavercdsn .. "\\"
				end
				slavercdsn = slavercdsn .. tmp_rcdsn
			end		
		end
	end
	
	jsonRoot["rcd sn"] = string.len(rcdsn)~=0 and rcdsn or "NA"	
	jsonRoot["slave rcd sn"] = slavercdsn
			
	local value = GeneralFun.getInfoBySeq(p,26)		if value == false then return end
	local baseVersion = value[webip]["baseVersion"]
	jsonRoot["rcdosVer"] = string.len(baseVersion)~=0 and baseVersion or "NA"
	
	local value = GeneralFun.getInfoBySeq(p,2)		if value == false then return end
	local patchVer = value[webip]["version"]
	patchVer = patchVer~=baseVersion and patchVer or ""
	jsonRoot["RK_PATCH_INFO"] = patchVer
	
	jsonRoot["score"] = ''..lua_getScore()
	jsonRoot["nopasscount"] = ''..(lua_getWarningCount() + lua_getFailedCount())
		
	local ids = {lua_getNotPassIDs()}
	local RK_WARN_ITEM = ''
	for i=1,#ids do
		local tmpID = ids[i]
		local desc = lua_getDescByID(tmpID)
		local tip = lua_getTipByID(tmpID,false)
		local tips = GeneralFun.stringSplit(tip,"\r\n")
		tip = ''
		for j=1,#tips do
			tip = tip .. "<tr><td>&nbsp;&nbsp;&nbsp;"
			tip = tip .. tips[j]
			tip = tip .. "</td></tr>"
		end
		RK_WARN_ITEM = RK_WARN_ITEM .. "<h4>" .. i .. "." .. desc .. "</h4>"
		RK_WARN_ITEM = RK_WARN_ITEM .. "<table>" .. tip .. "</table>"
	end
	jsonRoot["RK_WARN_ITEM"] = RK_WARN_ITEM
	
	local value = GeneralFun.getInfoBySeq(p,3)		if value == false then return end
	local netmode = 0
	if string.lower(value[webip]["deployMode"]) == "normal" then
		netmode = tonumber(value[webip]["localMode"])
	else
		netmode = 4
	end	
	
	local modes = {}
	modes[0] = "云桌面校园网云终端校园网"
	modes[1] = "云桌面校园网云终端私有网"
	modes[2] = "云桌面私有网云终端校园网"
	modes[3] = "云桌面私有网云终端私有网"
	modes[4] = "集群"
	
	jsonRoot["RK_NET_MODE"] = lua_UTF8ToChar(modes[netmode])
	
	local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return end
	
	jsonRoot["RK_RCD_WAN_IP"] = value[webip]["ip"]
	
	jsonRoot["RK_WAN_MASK"] = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,5)		if value == false then return end
	
	jsonRoot["RK_RCD_1ST_IP"] = value[webip]["ip"]
	
	local RK_RCD_2ND_IP = value[webip]["ip2"]
	if string.len(RK_RCD_2ND_IP) == 0 then
		RK_RCD_2ND_IP = "网卡工作模式：链路聚合模式"
	else
		RK_RCD_2ND_IP = "第二网口地址：" .. RK_RCD_2ND_IP
	end
	
	jsonRoot["RK_RCD_2ND_IP"] = lua_UTF8ToChar(RK_RCD_2ND_IP)
	
	jsonRoot["RK_RCD_MASK"] = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,8)		if value == false then return end
	jsonRoot["RK_RCD_GATEWAY"] = value[webip]["gateway"]
	
	local value = GeneralFun.getInfoBySeq(p,6)		if value == false then return end		
	jsonRoot["RK_RCD_DNS"] = value[webip]["dns"]
	
	local value = GeneralFun.getInfoBySeq(p,17)		if value == false then return end	
	jsonRoot["RK_TEACHER_IP"] = value[webip]["teacherIp"]
	
	--
	jsonRoot["RK_HALO_VERSION"] = lua_getDetectVersion()
	--
	jsonRoot["RK_RULE_VERSION"] = lua_getRuleVersion()
		
	local szdate=os.date("%Y.%m.%d %H:%M:%S")
	jsonRoot["RK_REPORT_TIME"] = szdate
	
	local value = GeneralFun.getInfoBySeq(p,13)		if value == false then return end		
	local RK_DESKTOP_INFO = ''
	for i=1,#value[webip] do
		RK_DESKTOP_INFO = RK_DESKTOP_INFO .. "<tr>" ..
							"<td>" .. value[webip][i]["vmHostName"] .. "</td>" ..
							"<td>" .. value[webip][i]["vmIp"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmHostName"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmNetType"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmIp"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmMask"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmGateway"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmDns"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmOsType"] .. "</td>" ..
							"<td>" .. value[webip][i]["tmOsVersion"] .. "</td>" ..						
							"</tr>"
	end
	jsonRoot["RK_DESKTOP_INFO"] = RK_DESKTOP_INFO
	
	
	
	local jsonStr = cjson.encode(jsonRoot)	
	local rpResult = lua_detectReport(jsonStr)
	
	if not rpResult then
		lua_addTip(p,"生成报告失败，请联系锐捷技术支持热线")
		lua_addResult(p,2)
		lua_addScore(p,-0)
	else
		lua_addResult(p,1)
	end
			
	lua_afterDetect(p)	
end
 
return detection