local detection = {}

local desc = '网络部署检测'
local cjson = require "cjson"
local GeneralFun = require ('data/rule/detection/GeneralFun')

--1.1、终端私网、虚机校园网模式：上网地址、两个物理网口地址、DNS
function detection06_mode1(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return false end
	local ip = value[webip]["ip"]
	local mask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,5)		if value == false then return false end
	local p1 = value[webip]["ip"]
	local p2 = value[webip]["ip2"]
	local netmask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,6)		if value == false then return false end
	local dns = value[webip]["dns"]
	
	local value = GeneralFun.getInfoBySeq(p,7)		if value == false then return false end
	local ip1 = value[webip]["ip"]
	local ip2 = value[webip]["ip2"]
	local netmask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,8)		if value == false then return false end
	local gateway = value[webip]["gateway"]
	
	--a、检测上网地址、掩码、网关是否配置，并且上网地址与网关同网段，若不对应，分数 -10分。提示1
	if string.len(ip) == 0 or string.len(mask) == 0 or string.len(gateway) == 0 or lua_isSameSubnet(ip,gateway,mask,netmask) == false then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"RCD网络配置错误或未配置上网地址")
	else
		lua_addResult(p,1)
	end
	--b、检测上网地址是否与两个云主机网口地址是否同网段，若同网段，分数：-10分。提示2
	if lua_isSameSubnet(ip,p1,mask,netmask) or lua_isSameSubnet(ip,p2,mask,netmask) then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"RCD网络配置错误，请核实该网络模式 上网地址与云主机网口地址配置规则")
	else
		lua_addResult(p,1)
	end
	--c、检测DNS未配置或者配置为8.8.8.8，分数-5分。提示3
	if string.len(dns) == 0 or dns == "8.8.8.8" then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"DNS设置不正确，请修改为本地运营商DNS或使用114.114.114.114")
	else
		lua_addResult(p,1)
	end
	
	return true
end

--1.2、终端、虚机校园网模式：两个网卡IP、网关、DNS
function detection06_mode2(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,6)		if value == false then return false end
	local dns = value[webip]["dns"]
	
	--a 检测DNS未配置或者配置为8.8.8.8，分数-5分。提示3
	if string.len(dns) == 0 or dns == "8.8.8.8" then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"DNS设置不正确，请修改为本地运营商DNS或使用114.114.114.114")
	else
		lua_addResult(p,1)
	end	
	return true
end

--1.3、终端、虚机私网模式：上网地址
function detection06_mode3(p)
	local webip = lua_getWebIP()
	local value = GeneralFun.getInfoBySeq(p,4)		if value == false then return false end
	local ip = value[webip]["ip"]
	local mask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,5)		if value == false then return false end
	local p1 = value[webip]["ip"]
	local p2 = value[webip]["ip2"]
	--local netmask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,6)		if value == false then return false end
	local dns = value[webip]["dns"]
	
	local value = GeneralFun.getInfoBySeq(p,7)		if value == false then return false end
	local ip1 = value[webip]["ip"]
	local ip2 = value[webip]["ip2"]
	local netmask = value[webip]["netmask"]
	
	local value = GeneralFun.getInfoBySeq(p,8)		if value == false then return false end
	local gateway = value[webip]["gateway"]	
		
	--a、检测上网地址、掩码、网关是否配置，并且上网地址与网关同网段，若不对应，分数 -10分。提示1
	if string.len(ip) == 0 or string.len(mask) == 0 or string.len(gateway) == 0 or lua_isSameSubnet(ip,gateway,mask,netmask) == false then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"RCD网络配置错误或未配置上网地址")
	else
		lua_addResult(p,1)
	end
	
	--b 检测云桌面地址是否配置为192.168.123.0 / 24网段，若不对应，分数 ： - 10分。提示4	
	local value = GeneralFun.getInfoBySeq(p,13)		if value == false then return false end
	local b = true
	local boxSize = #value[webip]		
	for i=1,boxSize do
		local ip = value[webip][i]["vmIp"]
		local mask = value[webip][i]["vmMask"]
		if string.find(ip,"192.168.123.") ~= 1 or mask ~= "255.255.255.0" then
			b = false
			break
		end
	end		
	
	if b == false then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"该网络部署模式,云桌面地址需填充为192.168.123.0/24网段")
	else
		lua_addResult(p,1)
	end
	
	return true
end

function detection.Detect()
	local webip = lua_getWebIP()
	local p = GeneralFun.getClassPointer()
	GeneralFun.writeFileUTF8("== "..desc.." ==")

	--1
	local value = GeneralFun.getInfoBySeq(p,3)		if value == false then return end
	local netcardMode = string.lower(value[webip]["netcardMode"])
	local deployMode = string.lower(value[webip]["deployMode"])
	local bBond = netcardMode == "bond"

	if deployMode == "normal" then
		local mode = value[webip]["localMode"]
		if mode == "1" then 
			local bRunOK = detection06_mode1(p)
			if not bRunOK then
				return
			end
		elseif mode == "0" then
			local bRunOK = detection06_mode2(p)
			if not bRunOK then
				return
			end
		elseif mode == "3" then
			local bRunOK = detection06_mode3(p)
			if not bRunOK then
				return
			end
		else
			-- todo. log unknow mode
		end
	else
		-- todo. log Cluster !!!
	end
	--2
	--2.1、RCD服务器两个物理网口link为非千兆或网口未连接，分数 - 10分  提示5	
	local value = GeneralFun.getInfoBySeq(p,9)		if value == false then return end
	local eth0 = string.lower(value[webip]["eth0"])
	local eth1 = string.lower(value[webip]["eth1"])
	-- state 0-ok 1-无连接 2-非千兆
	local state_eth0 = 0
	local state_eth1 = 0
	if eth0 == "na" then
		state_eth0 = 1
	elseif eth0 == "1000mb/s" then
		state_eth0 = 0
	else
		state_eth0 = 2
	end
	if eth1 == "na" then
		state_eth1 = 1
	elseif eth1 == "1000mb/s" then
		state_eth1 = 0
	else
		state_eth1 = 2
	end
	
	if state_eth0 == 0 and state_eth1 == 0 then
		lua_addResult(p,1)
	else
		-- 未连接
		local tip1 = ''
		if state_eth0 == 1 then
			tip1 = "物理网口1"
		end
		if state_eth1 == 1 then
			if string.len(tip1) ~= 0 then
				tip1 = tip1 .. "和"
			end
			tip1 = tip1 .. "物理网口2"
		end
		-- 云主机网口一未连接，请确认云主机网络连接状态
		if string.len(tip1) ~= 0 then
			lua_addResult(p,3)
			lua_addScore(p,-10)
			lua_addTip(p,"云主机"..tip1.."未连接，请确认云主机网络连接状态")
		end		
		
		-- 非千兆
		local tip2 = ''
		if state_eth0 == 2 then
			tip2 = "物理网口1"			
		end
		if state_eth1 == 2 then
			if string.len(tip2) ~= 0 then
				tip2 = tip2 .. "和"
			end
			tip2 = tip2 .. "物理网口2"
		end
		-- 云主机网口二非千兆，请检查网线与交换机是否为千兆协商模式
		if string.len(tip2) ~= 0 then
			lua_addResult(p,3)
			lua_addScore(p,-10)
			lua_addTip(p,"云主机"..tip2.."非千兆，请检查网线与交换机是否为千兆协商模式")
		end		
	end
		
	--2.2、检测到RCD服务器两个物理网口出现IP冲突情况 ，分数：-10分  提示6
	local value = GeneralFun.getInfoBySeq(p,10)		if value == false then return end
	local ip1 = value[webip]["ip"]
	local ip2 = value[webip]["ip2"]
	local conflict_ip = ''
	if ip1 ~= "no" then
		conflict_ip = "物理网口1"
	end
	if not (bBond or ip2 == "no") then
		if string.len(conflict_ip) == 0 then
			conflict_ip = "物理网口2"
		else
			conflict_ip = "两个物理网口都"
		end
	end
	
	if string.len(conflict_ip) == 0 then
		lua_addResult(p,1)		
	else
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"云主机"..webip.."，"..conflict_ip.."地址冲突，请确认网络地址分配情况")
	end	
		
	lua_afterDetect(p)
end
 
return detection