local detection = {}

local desc = '云桌面列表检测'
local cjson = require "cjson"
local GeneralFun = require ('data/rule/detection/GeneralFun')
 
function detection.Detect()
	local webip = lua_getWebIP()
	local p = GeneralFun.getClassPointer()
	GeneralFun.writeFileUTF8("== "..desc.." ==")
	
	local value = GeneralFun.getInfoBySeq(p,3)	if value == false then return end
	local localMode = value[webip]["localMode"]
		
	local value = GeneralFun.getInfoBySeq(p,6)	if value == false then return end
	local rcd_dns = value[webip]["dns"]
		
	local value = GeneralFun.getInfoBySeq(p,5)	if value == false then return end
	local ip1 = value[webip]["ip"]
	
	local value = GeneralFun.getInfoBySeq(p,8)	if value == false then return end
	local webgateway = value[webip]["gateway"]
	
	local value = GeneralFun.getInfoBySeq(p,2)	if value == false then return end
	local rcd_osver = value[webip]["version"]
	local config = GeneralFun.getConfig()
	local win_box_ver = config["boxVersion"]["windows"]
	local linux_serOsVer = config["boxVersion"]["linux"]["osver"]
	local linux_box_ver1 = config["boxVersion"]["linux"]["ver1"]
	local linux_box_ver2 = config["boxVersion"]["linux"]["ver2"]
	
	local value = GeneralFun.getInfoBySeq(p,13)	if value == false then return end
	local size = #value[webip]
	if size == 0 then
		lua_addResult(p,2)
		lua_addScore(p,0)
		lua_addTip(p,"未配置云桌面，请检查云桌面配置情况")
		lua_afterDetect(p)
		return
	end
	local bSamePort = true
	local bSamePort0 = true
	local bSameSubnet = true
	local bRelation = false
	local bvmDHCP = false
	local btmDHCP = false
	local bAnyOneEmpty = false
	local bNotSameDns = false
	local bAddressConflict = false
	local addressConflict = ""
	local address = webip .. "，"
	local bRedundancy = false
	local bNeedUpdate = false
	local needUpdateCount = 0
	local bHostNameConflict_vm = false
	local bHostNameConflict_tm = false
	local bHostNameConflict_all = false
	local hostNameConflict_all = ''
	local hostNameConflict_vm = ''
	local hostNameConflict_tm = ''
	local hostName_vm = ''
	local hostName_tm = ''
	local hostName_all = ''
	local bHostNameEmpty_tm = false
	for i=1,size do
		if string.len(value[webip][i]["tmIp"]) ~= 0 then
			-- 10、在终端私网模式下（1和3），需检测每台终端的网关必须是云主机第一个网口地址，若出现一台或以上配置不正确，扣10分，给出提示10
			-- 终端私网模式下，云终端的网关需配置为云主机第一个网口IP地址
			-- 修改，0模式也要求比较
			local tmGateway = value[webip][i]["tmGateway"]
			if bSamePort and (localMode == "1" or localMode == "3") then
				if ip1 ~= tmGateway then
					bSamePort = false
				end
			elseif bSamePort0 and localMode == "0" then
				if webgateway ~= tmGateway then
					bSamePort0 = false
				end
			end
		
			-- 9、在网络模式0(终端和云桌面校园网模式)，检测终端和云桌面是否同网段，若不同网段，扣10分，并停止cm-box检测，给出提示：
			--	云终端与云桌面属于不同网段，请确认网络部署规划
			if bSameSubnet and localMode == "0" then
				--not lua_isSameSubnet(box_tearcherIP,p1,box_mask,netmask) then
				local tmIp = value[webip][i]["tmIp"]
				local tmMask = value[webip][i]["tmMask"]
				local vmIp = value[webip][i]["vmIp"]
				local vmMask = value[webip][i]["vmMask"]
				if string.len(tmIp) > 0 and string.len(tmMask) > 0 and string.len(vmIp) > 0 and string.len(vmMask) > 0 then
					if not lua_isSameSubnet(tmIp,vmIp,tmMask,vmMask) then
						bSameSubnet = false
					end
				end				
			end
		
			bRelation = true
			--1、云桌面未填充IP，推荐使用静态IP获取模式， 分数： - 10分
			local vmIp = value[webip][i]["vmIp"]
			if string.len(vmIp) == 0 then
				bvmDHCP = true
			end
			--2、云桌面列表中获取终端信息中，网络模式为空或者动态模式，分数：-10分
			local tmNetType = string.lower(value[webip][i]["tmNetType"])
			if string.len(tmNetType) == 0 or tmNetType == "dhcp" then
				btmDHCP = true
			end
			--3.云桌面列表中获取终端信息中，掩码、网关、DNS 中有任意项为空，分数：-10分
			local tmMask = value[webip][i]["tmMask"]
			local tmGateway = value[webip][i]["tmGateway"]
			local tmDns = value[webip][i]["tmDns"]
			if string.len(tmMask) == 0 or string.len(tmGateway) == 0 or string.len(tmDns) == 0 then
				bAnyOneEmpty = true
			end
			--4、云桌面列表中获取终端信息中，云终端DNS与云主机不一致，分数： - 5分
			local tmDns = value[webip][i]["tmDns"]
			if tmDns ~= rcd_dns then
				bNotSameDns = true
			end
			--5、检测云桌面、云终端、RCD服务器之间出现地址冲突，分数：-10分
			local vmIp = value[webip][i]["vmIp"]
			local tmIp = value[webip][i]["tmIp"]
			if string.len(vmIp) ~= 0 then
				if string.find(address,vmIp.."，") == nil then
					address = address .. vmIp .. "，"
				else
					bAddressConflict = true
					if string.len(addressConflict) ~= 0 then
						addressConflict = addressConflict .. "，"
					end
					addressConflict = addressConflict .. vmIp
				end
			end
			if string.len(tmIp) ~= 0 then
				if string.find(address,tmIp.."，") == nil then
					address = address .. tmIp .. "，"
				else
					bAddressConflict = true
					if string.len(addressConflict) ~= 0 then
						addressConflict = addressConflict .. "，"
					end
					addressConflict = addressConflict .. tmIp
				end
			end
			
			--7、检测到终端系统版本非2.0以上版本，分数 - 10分		
			local box_ver = value[webip][i]["tmOsVersion"]
			local box_type = string.lower(value[webip][i]["tmOsType"])
			if box_type ~= "android" then
				if box_type == "windows" then
					if lua_compareVer(box_ver,win_box_ver) == 3 then
						bNeedUpdate = true
						needUpdateCount = needUpdateCount + 1
					end
				elseif box_type == "linux" then
					if lua_compareVer(rcd_osver,linux_serOsVer) == 2 then
						if lua_compareVer(box_ver,linux_box_ver1) == 3 then
							bNeedUpdate = true
							needUpdateCount = needUpdateCount + 1
						end						
					else
						if lua_compareVer(box_ver,linux_box_ver2) == 3 then
							bNeedUpdate = true
							needUpdateCount = needUpdateCount + 1
						end
					end
				end
			end
			-- 8、终端与虚机主机名冲突检测		
			--local bHostNameConflict_vm = false
			--local bHostNameConflict_tm = false
			--local hostNameConflict_all = ''
			--local hostNameConflict_vm = ''
			--local hostNameConflict_tm = ''
			--local hostName_all = ''
			--local hostName_vm = ''
			--local hostName_tm = ''
			--local bHostNameEmpty_tm = false			
			local vmHostName = value[webip][i]["vmHostName"]
			local tmHostName = value[webip][i]["tmHostName"]
			if string.len(vmHostName) ~= 0 then				
				if lua_find(hostName_vm,vmHostName.."，") ~= -1 then
					bHostNameConflict_vm = true
					hostNameConflict_vm = hostNameConflict_vm .. vmHostName.. "，"	
					hostNameConflict_all = hostNameConflict_all .. vmHostName.. "，"	
				elseif lua_find(hostName_tm,vmHostName.."，") ~= -1 then
					bHostNameConflict_all = true
					hostNameConflict_all = hostNameConflict_all .. vmHostName.. "，"	
				end	
				hostName_vm = hostName_vm  .. vmHostName .. "，"				
			end
			if string.len(tmHostName) ~= 0 then
				if lua_find(hostName_tm,tmHostName.."，") ~= -1 then
					bHostNameConflict_tm = true
					hostNameConflict_tm = hostNameConflict_tm .. tmHostName.. "，"	
					hostNameConflict_all = hostNameConflict_all .. tmHostName.. "，"	
				elseif lua_find(hostName_vm,tmHostName.."，") ~= -1 then
					bHostNameConflict_all = true
					hostNameConflict_all = hostNameConflict_all  .. tmHostName.. "，"	
				end
				hostName_tm = hostName_tm .. tmHostName .. "，"
			else
				bHostNameEmpty_tm = true				
			end
			
		-- 6、检测到有多余云桌面（无终端关联），分数： - 5分
		else	
			bRedundancy = true			
		end
	end
	
	if not bRelation then		
		lua_addResult(p,2)
		lua_addScore(p,0)
		lua_addTip(p,"云课堂未关联任何云终端，请确认云课堂部署情况")
		lua_afterDetect(p)
		return
	end
	
	-- 1	bvmDHCP
	if bvmDHCP then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"云桌面推荐使用静态IP模式，以防止出现地址冲突")
	else
		lua_addResult(p,1)
	end
	-- 2	btmDHCP
	if btmDHCP then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"云终端推荐使用静态IP模式，请使用classmannager对云终端进行一键部署")
	else
		lua_addResult(p,1)
	end
	-- 3	bAnyOneEmpty
	if bAnyOneEmpty then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"云终端配置不完整，请使用classmannager对云终端进行一键部署")
	else
		lua_addResult(p,1)
	end
	-- 4	bNotSameDns
	if bNotSameDns then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"云终端DNS配置与RCD服务器不一致，请确认云终端DNS配置是否正确")
	else
		lua_addResult(p,1)
	end
	-- 5	bAddressConflict
	if bAddressConflict then
		local tmpTip = "云桌面、云终端、RCD服务器之间出现地址冲突，请检查网络地址规划"
		local conflict = ''
		if string.len(addressConflict)~=0 then
			conflict = conflict .. "; 冲突的IP有：" .. addressConflict
		end	
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,tmpTip.."(详情见体检报告)",tmpTip.. conflict)
	else
		lua_addResult(p,1)
	end
	-- 6	bRedundancy
	if bRedundancy then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"云桌面配置出现冗余，请删除多余云桌面")
	else
		lua_addResult(p,1)
	end
	-- 7	bNeedUpdate	needUpdateCount
	if bNeedUpdate then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"有"..needUpdateCount.."台终端需要升级系统版本")
	else
		lua_addResult(p,1)
	end	
	-- 8、终端与虚机主机名冲突检测	
	
	
			--local bHostNameConflict_vm = false
			--local bHostNameConflict_tm = false
			--local hostNameConflict_all = ''
			--local hostNameConflict_vm = ''
			--local hostNameConflict_tm = ''
			--local hostName_all = ''
			--local hostName_vm = ''
			--local hostName_tm = ''
			--local bHostNameEmpty_tm = false	
	
	if bHostNameEmpty_tm then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		lua_addTip(p,"未配置云终端主机名，请使用classmannager对云终端进行一键部署配置")
	else
		lua_addResult(p,1)
	end
	
	if	bHostNameConflict_tm or bHostNameConflict_vm or bHostNameConflict_all then
		lua_addResult(p,3)
		lua_addScore(p,-5)
		if bHostNameConflict_all then
			local tmp_tip = "云桌面与云终端之间存在主机名冲突的情况，请检测云桌面与云终端之间配置情况"			
			lua_addTip(p,tmp_tip.."(详情见体检报告)",tmp_tip.."，冲突的主机名有："..hostNameConflict_all)		
		else
			if bHostNameConflict_tm then
				local tmp_tip = "云终端之间存在主机名冲突的情况，请检测云终端配置情况"				
				lua_addTip(p,tmp_tip.."(详情见体检报告)",tmp_tip.."，冲突的主机名有："..hostNameConflict_tm)	
			end
			if bHostNameConflict_vm then
				local tmp_tip = "云桌面之间存在主机名冲突的情况，请检测云桌面配置情况"				
				lua_addTip(p,tmp_tip.."(详情见体检报告)",tmp_tip.."，冲突的主机名有："..hostNameConflict_tm)	
			end		
		end
	else
		lua_addResult(p,1)	
	end
	
	-- 10	-- 终端私网模式下，云终端的网关需配置为云主机第一个网口IP地址
	if not bSamePort then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"终端私网模式下，云终端的网关需配置为云主机第一个网口IP地址")
	else
		lua_addResult(p,1)
	end
	if not bSamePort0 then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"云主机该网络模式下，云终端的网关需配置与云主机网关一致")
	else
		lua_addResult(p,1)
	end
	
	-- 9 bSameSubnet			--	云终端与云桌面属于不同网段，请确认网络部署规划
	if not bSameSubnet then
		lua_addResult(p,3)
		lua_addScore(p,-10)
		lua_addTip(p,"云终端与云桌面属于不同网段，请确认网络部署规划")
		lua_removeDetection(p,11)
	else
		lua_addResult(p,1)
	end
	
	
	lua_afterDetect(p)
end
 
return detection